<?php

class WebserviceSpecificManagementRetinoOrder implements WebserviceSpecificManagementInterface
{
    protected $data;

    /** @var WebserviceOutputBuilder */
    protected $objOutput;
    protected $output;

    /** @var WebserviceRequest */
    protected $wsObject;

    /* ------------------------------------------------
     * GETTERS & SETTERS
     * ------------------------------------------------ */

    /**
     * @param WebserviceOutputBuilderCore $obj
     *
     * @return WebserviceSpecificManagementInterface
     */
    public function setObjectOutput(WebserviceOutputBuilderCore $obj)
    {
        $this->objOutput = $obj;

        return $this;
    }

    public function setWsObject(WebserviceRequestCore $obj)
    {
        $this->wsObject = $obj;

        return $this;
    }

    public function getWsObject()
    {
        return $this->wsObject;
    }

    public function getObjectOutput()
    {
        return $this->objOutput;
    }

    public function setUrlSegment($segments)
    {
        $this->urlSegment = $segments;

        return $this;
    }

    public function getUrlSegment()
    {
        return $this->urlSegment;
    }

    /**
     * All webservice logic is here
     */
    public function manage()
    {
        try {
            // if (!isset($this->wsObject->urlSegment[1]) || !is_int($this->wsObject->urlSegment[1])) {
            //     throw new WebserviceException('You have to specify resource id', array(100, 400));
            // }
            $order_keys = array(
                'id',
                'reference',
                'date_add',
                'total_paid',
                'total_paid_tax_incl',
                'total_paid_tax_excl',
                'total_paid_real',
                'total_products',
                'total_products_wt',
                'total_shipping',
                'total_shipping_tax_incl',
                'total_shipping_tax_excl',
            );
            $state_keys = array(
                'id_order_state',
                'name'
            );
            $product_keys = array(
                'product_id',
                'product_name',
                'product_quantity',
                'quantity',
                'price',
                'product_price',
                'total_price_tax_incl',
                'total_price_tax_excl',
                'unit_price_tax_incl',
                'unit_price_tax_excl',
            );
            $currency_keys = array('iso_code');
            $customer_keys = array(
                'firstname',
                'lastname',
                'email'
            );
            $address_keys = array(
                'country',
                'company',
                'lastname',
                'firstname',
                'address1',
                'address2',
                'postcode',
                'city',
                'phone'
            );

            $order_id = $this->wsObject->urlSegment[1];

            $obj = new Order($order_id);
            $products = array();

            foreach ($obj->getProducts() as $product) {
                $products[] = $this->extractKeys($product, $product_keys);
            }

            $current_state = $this->extractKeys($obj->getCurrentStateFull(1), $state_keys);
            $currency = $this->extractKeys(new Currency($obj->id_currency), $currency_keys);
            $customer = $this->extractKeys(new Customer($obj->id_customer), $customer_keys);
            $delivery_address = $this->extractKeys(new Address($obj->id_address_delivery), $address_keys);
            $invoice_address = $this->extractKeys(new Address($obj->id_address_invoice), $address_keys);

            $data = $this->extractKeys($obj, $order_keys);
            $data['products'] = $products;
            $data['current_state'] = $current_state;
            $data['currency'] = $currency;
            $data['customer'] = $customer;
            $data['delivery_address'] = $delivery_address;
            $data['invoice_address'] = $invoice_address;

            // save last successful import datetime
            $retino_module = Module::getInstanceByName('retino');
            $retino_module->setConfigValues(
                array(
                    'last_successful_import_readable' => date('l jS \of F Y h:i:s A'),
                    'last_successful_import_timestamp' => time(),
                ),
                true
            );

            $this->data = $data;
        } catch (\Throwable $e) {
            throw new WebserviceException($e->getMessage(), array(100, 400));
        }
    }

    /**
     * This must be return a string with specific values as WebserviceRequest expects.
     *
     * @return string
     */
    public function getContent()
    {
        return json_encode($this->data);
    }

    private function extractKeys($obj, $keys)
    {
        $obj = (array) json_decode(json_encode($obj));

        $result = array();
        foreach ($keys as $key) {
            if (array_key_exists($key, $obj)) {
                $result[$key] = $obj[$key];
            }
        }
        return $result;
    }
}
